@@ -1,6 +1,6 @@
-/* $Id: dialogs.c 1593 2004-11-24 10:03:30Z kelnos $ */
-/*-
- * Copyright (c) 2003 Jasper Huijsmans <huysmans@users.sourceforge.net>
+/* $Id: dialogs.c 21424 2006-05-01 15:09:21Z benny $ */
+/*
+ * Copyright (c) 2003,2005 Jasper Huijsmans <jasper@xfce.org>
  * Copyright (c) 2003 Benedikt Meurer <benny@xfce.org>
  *
  * This library is free software; you can redistribute it and/or
@@ -45,6 +45,7 @@
 #include <gtk/gtk.h>
 
 #include "dialogs.h"
+#include "xfce-gtk-extensions.h"
 
 #ifndef PATH_MAX
 #define DEFAULT_LENGTH 1024
@@ -268,7 +269,7 @@ message_dialog (GtkMessageType type, const gchar * message)
 				  GTK_DIALOG_MODAL,
 				  type, GTK_BUTTONS_CLOSE, message);
 
-    gtk_window_set_position (GTK_WINDOW (dlg), GTK_WIN_POS_CENTER);
+    xfce_gtk_window_center_on_monitor_with_pointer (GTK_WINDOW (dlg));
     gtk_dialog_run (GTK_DIALOG (dlg));
     gtk_widget_destroy (dlg);
 }
@@ -453,9 +454,9 @@ xfce_confirm (const gchar *text,
     gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button,
                                   GTK_RESPONSE_YES);
 
-    gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
     gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER);
 
+    xfce_gtk_window_center_on_monitor_with_pointer (GTK_WINDOW (dialog));
     response = gtk_dialog_run (GTK_DIALOG (dialog));
     gtk_widget_hide (dialog);
     gtk_widget_destroy (dialog);
@@ -466,22 +467,253 @@ xfce_confirm (const gchar *text,
         return FALSE;
 }
 
+static GtkWidget *
+xfce_message_dialog_new_valist(GtkWindow *parent,
+                               const gchar *title,
+                               const gchar *icon_id,
+                               const gchar *primary_text,
+                               const gchar *secondary_text,
+                               const gchar *first_button_type,
+                               va_list args)
+{
+    GtkWidget *dialog, *label;
+    GtkWidget *hbox, *align;
+    GtkWidget *image;
+    gchar *markup = NULL;
+
+    /* create the dialog */
+    dialog = gtk_dialog_new ();
+
+    gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
+    gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
+    gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
+
+    /* set title if needed */
+    if (title)
+        gtk_window_set_title (GTK_WINDOW (dialog), title);
+    else
+        gtk_window_set_title (GTK_WINDOW (dialog), "");
+
+    /* add image */
+    hbox = gtk_hbox_new (FALSE, 12);
+
+    if (icon_id)
+    {
+        image = gtk_image_new_from_stock (icon_id, GTK_ICON_SIZE_DIALOG);
+        gtk_misc_set_alignment (GTK_MISC (image), 0, 0);
+
+        gtk_box_pack_start (GTK_BOX (hbox), image, TRUE, FALSE, 0);
+    }
+
+    /* add the text */
+    markup = NULL;
+
+    if (primary_text && secondary_text)
+    {
+        markup = g_strdup_printf ("<span weight='bold' size='large'>%s</span>"
+                                  "\n\n%s", primary_text, secondary_text);
+    }
+    else if (primary_text)
+    {
+        markup =
+            g_strdup_printf ("<span weight='bold' size='large'>%s</span>",
+                             primary_text);
+    }
+    else
+    {
+        markup = g_strdup (secondary_text);
+    }
+
+    label = gtk_label_new (NULL);
+
+    gtk_label_set_markup (GTK_LABEL (label), markup);
+    gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+    gtk_label_set_selectable (GTK_LABEL (label), TRUE);
+
+    gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
+
+    g_free (markup);
+
+    gtk_widget_show_all (hbox);
+    gtk_box_pack_start (GTK_BOX ((GTK_DIALOG (dialog))->vbox), hbox,
+                        FALSE, FALSE, 0);
+
+    align = gtk_alignment_new (0, 0, 0, 0);
+    gtk_widget_set_size_request (align, 12, 12);
+    gtk_widget_show (align);
+    gtk_box_pack_start (GTK_BOX ((GTK_DIALOG (dialog))->vbox), align,
+                        FALSE, FALSE, 0);
+
+    /* sizing according to GNOME HIG, 
+     * except with a border of 8 instead of 12 */
+    gtk_container_set_border_width (GTK_CONTAINER (dialog), 2);
+    gtk_container_set_border_width (GTK_CONTAINER (hbox), 6);
+
+    /* add the buttons */
+    if (first_button_type)
+    {
+        const gchar *text;
+        gint response_id;
+
+        text = first_button_type;
+
+        while (text != NULL)
+        {
+            GtkWidget *button;
+
+            if (strcmp (text, XFCE_CUSTOM_PIXBUF_BUTTON) == 0)
+            {
+                GdkPixbuf *icon, *scaled;
+                GtkWidget *align, *image, *hbox, *label;
+                gint w, h;
+
+                text = va_arg (args, gchar *);
+                icon = va_arg (args, GdkPixbuf *);
+                response_id = va_arg (args, int);
+
+                button = gtk_button_new ();
+                label = gtk_label_new_with_mnemonic (text);
+                gtk_label_set_mnemonic_widget (GTK_LABEL (label), button);
+
+                gtk_icon_size_lookup (GTK_ICON_SIZE_BUTTON, &w, &h);
+
+                if ((gdk_pixbuf_get_width (icon) != w) &&
+                    (gdk_pixbuf_get_height (icon) != h))
+                {
+                    scaled = gdk_pixbuf_scale_simple (icon, w, h,
+                                                      GDK_INTERP_BILINEAR);
+
+                    g_object_unref (G_OBJECT (icon));
+                    icon = scaled;
+                }
+
+                image = gtk_image_new_from_pixbuf (icon);
+
+                hbox = gtk_hbox_new (FALSE, 2);
+                align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
+
+                gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
+                gtk_box_pack_end (GTK_BOX (hbox), label, FALSE, FALSE, 0);
+
+                gtk_container_add (GTK_CONTAINER (button), align);
+                gtk_container_add (GTK_CONTAINER (align), hbox);
+                gtk_widget_show_all (align);
+            }
+            else if (strcmp (text, XFCE_CUSTOM_STOCK_BUTTON) == 0)
+            {
+                gchar *icon_stock;
+
+                text = va_arg (args, gchar *);
+                icon_stock = va_arg (args, gchar *);
+                response_id = va_arg (args, int);
+
+                button = xfce_create_mixed_button (icon_stock, text);
+            }
+            else if (strcmp (text, XFCE_CUSTOM_BUTTON) == 0)
+            {
+                text = va_arg (args, gchar *);
+                response_id = va_arg (args, int);
+
+                button = gtk_button_new_with_label (text);
+            }
+            else
+            {
+                response_id = va_arg (args, int);
+
+                button = gtk_button_new_from_stock (text);
+            }
+
+            gtk_widget_show (button);
+            gtk_dialog_add_action_widget (GTK_DIALOG (dialog), button,
+                                          response_id);
+
+            text = va_arg (args, gchar *);
+        }
+    }
+
+    if (parent)
+    {
+        gtk_window_set_transient_for (GTK_WINDOW (dialog), parent);
+        gtk_window_set_position (GTK_WINDOW (dialog),
+                                 GTK_WIN_POS_CENTER_ON_PARENT);
+    }
+    else
+    {
+        xfce_gtk_window_center_on_monitor_with_pointer (GTK_WINDOW (dialog));
+    }
+    
+    return dialog;
+}
+
 /**
  * xfce_message_dialog:
- * @parent: Transient parent of the dialog, or %NULL
- * @title: Title of the dialog, or %NULL
- * @icon_id: Gtk stock icon to show in the dialog
- * @primary_text: Text shown in large and bolded font
- * @secondary_text: Text shown in normal font
- * @first_button_type: Type of the first button, or %NULL
- * @Varargs: %NULL ended list of parameters and response ID
+ * @parent            : Transient parent of the dialog, or %NULL
+ * @title             : Title of the dialog, or %NULL
+ * @icon_id           : Gtk stock icon to show in the dialog
+ * @primary_text      : Text shown in large bold font
+ * @secondary_text    : Text shown in normal font
+ * @first_button_type : Type of the first button, or %NULL
+ * @Varargs           : %NULL ended list of parameters and response ID
  *
- * xfce_message_dialog() allows you to easily create a dialog respecting GNOME HIG;
- * it accepts gtk stock buttons, or custom buttons using a gtk stock icon and text or a gdkpixbuf and text
- * or just a text. It runs a modal message dialog with transient parent @parent (or %NULL for none) 
- * and title @title (or %NULL for a default title) containing the icon @icon, large and bolded formatted @primary_text,
- * normal formatted @secondary_text and some buttons. The buttons are defined by first_button_type and the next arguments in
- * following format @type, @param1[, @param2, @param3]; there are four types of button :
+ * xfce_message_dialog() creates a dialog widget as in
+ * xfce_message_dialog_new(), and then runs the dialog as a modal dialog and
+ * returns the response id selected by the user.
+ *
+ * See xfce_message_dialog_new() for more information.
+ *
+ * Returns: The selected response id.
+ **/
+gint
+xfce_message_dialog(GtkWindow *parent,
+                    const gchar *title,
+                    const gchar *icon_id,
+                    const gchar *primary_text,
+                    const gchar *secondary_text,
+                    const gchar *first_button_type,
+                    ...)
+{
+    GtkWidget *dlg;
+    va_list args;
+    gint response;
+    
+    va_start(args, first_button_type);
+    
+    dlg = xfce_message_dialog_new_valist(parent, title, icon_id,
+                                         primary_text, secondary_text,
+                                         first_button_type, args);
+    
+    va_end(args);
+    
+    /* show dialog and return */
+    response = gtk_dialog_run(GTK_DIALOG(dlg));
+    gtk_widget_hide(dlg);
+    gtk_widget_destroy(dlg);
+
+    return response;
+}
+
+/**
+ * xfce_message_dialog_new:
+ * @parent            : Transient parent of the dialog, or %NULL
+ * @title             : Title of the dialog, or %NULL
+ * @icon_id           : Gtk stock icon to show in the dialog
+ * @primary_text      : Text shown in large bold font
+ * @secondary_text    : Text shown in normal font
+ * @first_button_type : Type of the first button, or %NULL
+ * @Varargs           : %NULL ended list of parameters and response ID
+ *
+ * xfce_message_dialog_new() allows you to easily create a dialog respecting 
+ * the GNOME HIG; it accepts gtk stock buttons, or custom buttons using a 
+ * gtk stock icon and text or a #GdkPixbuf and text, or just text. 
+ * 
+ * It returns a message dialog with transient parent @parent (or %NULL 
+ * for none) and title @title (or %NULL for a default title) containing 
+ * the icon @icon, large and bold @primary_text, normal @secondary_text and 
+ * some buttons. 
+ *
+ * The buttons are defined by first_button_type and the next arguments in
+ * following format @type, @param1[, @param2, @param3]; there are four types 
+ * of button :
  * <itemizedlist>
  *  <listitem>
  *  <para>
@@ -490,183 +722,57 @@ xfce_confirm (const gchar *text,
  *  </listitem>
  *  <listitem>
  *  <para>
- *  %XFCE_CUSTOM_BUTTON : @param1 is used for the button text and @param2 for the the response ID, @param3 isn't used.
+ *  %XFCE_CUSTOM_BUTTON : @param1 is used for the button text and @param2 for 
+ *  the the response ID, @param3 isn't used.
  *  </para>
  *  </listitem>
  *  <listitem>
  *  <para>
- *  %XFCE_CUSTOM_STOCK_BUTTON : @param1 is used for the button text, @param2 for the gtk stock icon name and @param3 for the response ID.
+ *  %XFCE_CUSTOM_STOCK_BUTTON : @param1 is used for the button text, @param2 
+ *  for the gtk stock icon name and @param3 for the response ID.
  *  </para>
  *  </listitem>
  *  <listitem>
  *  <para>
- *  %XFCE_CUSTOM_PIXBUF_BUTTON : @param1 is used for the button text, @param2 for the GdkPixbuf pointer and @param3 for the response ID.
+ *  %XFCE_CUSTOM_PIXBUF_BUTTON : @param1 is used for the button text, @param2 
+ *  for the GdkPixbuf pointer and @param3 for the response ID.
  *  </para>
  *  </listitem>
  * </itemizedlist>  
  * Here's a simple example:
  * <informalexample><programlisting>
- *  gint response = xfce_message_dialog (parent, "Question",
- *                                       GTK_STOCK_DIALOG_QUESTION,
- *                                       "Do you want to save before closing the menu ?",
- *                                       "The menu has been modified, do you want to save it before quitting ?",
- *                                       XFCE_CUSTOM_STOCK_BUTTON, "Forget modifications", GTK_STOCK_QUIT, GTK_RESPONSE_NO,
- *                                       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
- *                                       GTK_STOCK_SAVE, GTK_RESPONSE_YES, NULL);
+ *  GtkWidget *dlg = xfce_message_dialog (parent, "Question",
+ *                                        GTK_STOCK_DIALOG_QUESTION,
+ *                                        "There are unsaved modifications",
+ *                                        "The menu has been modified, do you want to save it before quitting?",
+ *                                        XFCE_CUSTOM_STOCK_BUTTON, "Forget modifications", 
+ *                                        GTK_STOCK_QUIT, GTK_RESPONSE_NO,
+ *                                        GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+ *                                        GTK_STOCK_SAVE, GTK_RESPONSE_YES, 
+ *                                        NULL);
  * </programlisting></informalexample>
  *
- * Return value: selected response ID.
+ * Return value: A new #GtkDialog.
  **/
-gint
-xfce_message_dialog (GtkWindow *parent,
-		     const gchar *title,
-		     const gchar *icon_id,
-		     const gchar *primary_text,
-		     const gchar *secondary_text,
-		     const gchar *first_button_type, ...)
+GtkWidget *
+xfce_message_dialog_new(GtkWindow *parent,
+                        const gchar *title,
+                        const gchar *icon_id,
+                        const gchar *primary_text,
+                        const gchar *secondary_text,
+                        const gchar *first_button_type,
+                        ...)
 {
-    GtkWidget *dialog, *label;
-    GtkWidget *hbox;
-    GtkWidget *image;
-    gchar *markup = NULL;
-    gint response = GTK_RESPONSE_NONE;
+    GtkWidget *dlg;
     va_list args;
-
-    /* create the dialog */
-    dialog = gtk_dialog_new();
-
-    if (parent)
-        gtk_window_set_transient_for (GTK_WINDOW (dialog), parent);
-    gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_CENTER_ON_PARENT);
-    gtk_window_set_modal (GTK_WINDOW (dialog), TRUE);
-    gtk_window_set_resizable(GTK_WINDOW (dialog), FALSE);
-    gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
-    gtk_container_set_border_width(GTK_CONTAINER (dialog), 6);
-
-    /* set title if needed */
-    if (title)
-        gtk_window_set_title (GTK_WINDOW (dialog), title);
-    else
-        gtk_window_set_title (GTK_WINDOW (dialog), "Message dialog");
-
-    /* add image */
-    hbox = gtk_hbox_new (FALSE, 6);
-
-    if (icon_id){
-        image = gtk_image_new_from_stock (icon_id, GTK_ICON_SIZE_DIALOG);
-	gtk_misc_set_alignment (GTK_MISC (image), 0.5, 0.5);
-  
-	gtk_box_pack_start (GTK_BOX (hbox), image, TRUE, FALSE, 0);
-    }
-
-    /* add the text */
-    markup = g_strdup ("");
-    if (primary_text){
-        gchar *temp;
-	temp = g_strdup_printf ("<span weight='bold' size='large'>%s</span>\n", primary_text);
-	markup = g_strconcat (markup, temp, NULL);
-	g_free (temp);
-    }
-
-    if (secondary_text){
-        gchar *temp;
-	temp = g_strdup_printf ("\n%s\n", secondary_text);
-	markup = g_strconcat (markup, temp, NULL);
-	g_free (temp);
-    }
-
-    label = gtk_label_new (primary_text);
     
-    gtk_label_set_markup (GTK_LABEL (label), markup);
-    gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
-    gtk_label_set_selectable (GTK_LABEL (label), TRUE);
+    va_start(args, first_button_type);
+    dlg = xfce_message_dialog_new_valist(parent, title, icon_id,
+                                         primary_text, secondary_text,
+                                         first_button_type, args);
+    va_end(args);
     
-    gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
-
-    g_free (markup);
-
-    gtk_widget_show_all (hbox);
-    gtk_box_pack_start (GTK_BOX ((GTK_DIALOG (dialog))->vbox), hbox, FALSE, FALSE, 0);
-
-    /* add the buttons */
-    if (first_button_type){
-        const gchar* text;
-        gint response_id;
-
-	va_start (args, first_button_type);
-
-	text = first_button_type;
-
-	while (text != NULL){
-	    GtkWidget *button;
-
-	    if (strcmp (text, XFCE_CUSTOM_PIXBUF_BUTTON) == 0){
-	        GdkPixbuf *icon, *scaled;
-		GtkWidget *align, *image, *hbox, *label;
-		gint w, h;
-
-		text = va_arg (args, gchar*);
-		icon = va_arg (args, GdkPixbuf*);
-		response_id = va_arg (args, int);
-
-		button = gtk_button_new ();
-		label = gtk_label_new_with_mnemonic (text);
-		gtk_label_set_mnemonic_widget (GTK_LABEL (label), button);
-
-		gtk_icon_size_lookup(GTK_ICON_SIZE_BUTTON, &w, &h);
-
-		if ( (gdk_pixbuf_get_width (icon) != w) && (gdk_pixbuf_get_height (icon) != h) ){
-		    scaled = gdk_pixbuf_scale_simple (icon, w, h, GDK_INTERP_BILINEAR);
-		    
-		    g_object_unref (G_OBJECT (icon));
-		    icon = scaled;
-		}
-
-		image = gtk_image_new_from_pixbuf (icon);
-
-		hbox = gtk_hbox_new (FALSE, 2);
-		align = gtk_alignment_new (0.5, 0.5, 0.0, 0.0);
-
-		gtk_box_pack_start (GTK_BOX (hbox), image, FALSE, FALSE, 0);
-		gtk_box_pack_end (GTK_BOX (hbox), label, FALSE, FALSE, 0);
-
-		gtk_container_add (GTK_CONTAINER (button), align);
-		gtk_container_add (GTK_CONTAINER (align), hbox);
-		gtk_widget_show_all (align);
-	    }else if (strcmp (text, XFCE_CUSTOM_STOCK_BUTTON) == 0){
-	        gchar *icon_stock;
-
-	        text = va_arg (args, gchar*);
-	        icon_stock = va_arg (args, gchar*);
-	        response_id = va_arg (args, int);
-	  
-	        button = xfce_create_mixed_button (icon_stock, text);
-	    }else if(strcmp (text, XFCE_CUSTOM_BUTTON) == 0){
-	        text = va_arg (args, gchar*);
-	        response_id = va_arg (args, int);
-
-	        button = gtk_button_new_with_label (text);
-	    }else{
-	        response_id = va_arg (args, int);
-
-		button = gtk_button_new_from_stock (text);
-	    }
-
-	  gtk_widget_show (button);
-	  gtk_dialog_add_action_widget (GTK_DIALOG(dialog), button, response_id);
-
-	  text = va_arg (args, gchar*);	  
- 	}
-  
-	va_end (args);
-    }
-  
-    /* show dialog and return */
-    response = gtk_dialog_run (GTK_DIALOG (dialog));
-    gtk_widget_hide (dialog);
-    gtk_widget_destroy (dialog); 
-      
-    return response;
+    return dlg;
 }
 
 /* command execution with error dialog */
